//------------------------------------------------------------------------------
// emNetwalkPanel.cpp
//
// Copyright (C) 2010 Oliver Hamann.
//
// Homepage: http://eaglemode.sourceforge.net/
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License version 3 as published by the
// Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License version 3 for
// more details.
//
// You should have received a copy of the GNU General Public License version 3
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include <emNetwalk/emNetwalkPanel.h>
#include <emNetwalk/emNetwalkControlPanel.h>


emNetwalkPanel::emNetwalkPanel(
	ParentArg parent, const emString & name, emNetwalkModel * fileModel
)
	: emFilePanel(parent,name,fileModel,true)
{
	Mdl=fileModel;
	HaveControlPanel=IsVFSGood();
	Scrolling=false;
	PrepareTransformation();
	AddWakeUpSignal(Mdl->GetChangeSignal());
	AddWakeUpSignal(GetVirFileStateSignal());
}


emNetwalkPanel::~emNetwalkPanel()
{
}


emString emNetwalkPanel::GetTitle()
{
	return "Netwalk";
}


void emNetwalkPanel::GetEssenceRect(
	double * pX, double * pY, double * pW, double * pH
)
{
	*pX=EssenceX;
	*pY=EssenceY;
	*pW=EssenceW;
	*pH=EssenceH;
}


bool emNetwalkPanel::Cycle()
{
	bool vfsGood;

	if (IsSignaled(GetVirFileStateSignal())) {
		vfsGood=IsVFSGood();
		if (HaveControlPanel!=vfsGood) {
			HaveControlPanel=vfsGood;
			InvalidateControlPanel();
		}
		if (!vfsGood && Scrolling) {
			Scrolling=false;
			InvalidateCursor();
		}
	}

	if (
		IsSignaled(GetVirFileStateSignal()) ||
		IsSignaled(Mdl->GetChangeSignal())
	) {
		PrepareTransformation();
		InvalidatePainting();
	}

	return emFilePanel::Cycle();
}


void emNetwalkPanel::Notice(NoticeFlags flags)
{
	emFilePanel::Notice(flags);
	if (flags&NF_LAYOUT_CHANGED) {
		PrepareTransformation();
		InvalidatePainting();
	}
}


void emNetwalkPanel::Input(
	emInputEvent & event, const emInputState & state, double mx, double my
)
{
	int x,y;

	if (!IsViewed() || !IsVFSGood()) {
		emFilePanel::Input(event,state,mx,my);
		return;
	}

	if (Scrolling) {
		if (!state.Get(EM_KEY_LEFT_BUTTON) || !Mdl->IsBorderless()) {
			Scrolling=false;
			InvalidateCursor();
		}
		else {
			x=(int)floor((mx-ScrollX0)/DX+0.5);
			y=(int)floor((my-ScrollY0)/DY+0.5);
			if (x || y) {
				Mdl->Scroll(x,y);
				ScrollX0+=x*DX;
				ScrollY0+=y*DY;
			}
		}
	}

	if (event.IsLeftButton() && !Scrolling) {
		if (state.IsNoMod() || state.IsShiftMod()) {
			x=(int)floor((mx-X0)/DX);
			y=(int)floor((my-Y0)/DY);
			if (x>=0 && y>=0 && x<Mdl->GetWidth() && y<Mdl->GetHeight()) {
				if (state.IsNoMod()) Mdl->RotateLeft(x,y);
				else Mdl->RotateRight(x,y);
			}
		}
		else if (state.IsCtrlMod() && Mdl->IsBorderless()) {
			Scrolling=true;
			ScrollX0=mx;
			ScrollY0=my;
			InvalidateCursor();
		}
	}

	if (event.IsRightButton() && state.IsNoMod()) {
		x=(int)floor((mx-X0)/DX);
		y=(int)floor((my-Y0)/DY);
		if (x>=0 && y>=0 && x<Mdl->GetWidth() && y<Mdl->GetHeight()) {
			Mdl->MarkOrUnmark(x,y);
		}
	}

	if (event.IsKey(EM_KEY_N) && state.IsCtrlMod()) {
		try {
			Mdl->TrySetup(
				Mdl->GetWidth(),Mdl->GetHeight(),Mdl->IsBorderless(),
				Mdl->IsNoFourWayJunctions(),Mdl->GetComplexity()
			);
		}
		catch (emString errorMessage) {
			emTkDialog::ShowMessage(GetViewContext(),"Error",errorMessage);
		}
		event.Eat();
	}

	emFilePanel::Input(event,state,mx,my);
}


emCursor emNetwalkPanel::GetCursor()
{
	if (Scrolling) return emCursor::LEFT_RIGHT_UP_DOWN_ARROW;
	else return emFilePanel::GetCursor();
}


bool emNetwalkPanel::IsOpaque()
{
	if (IsVFSGood()) return true;
	else return emFilePanel::IsOpaque();
}


void emNetwalkPanel::Paint(const emPainter & painter, emColor canvasColor)
{
	int x,y,w,h,i;
	emString str;
	double t;

	if (!IsVFSGood()) {
		emFilePanel::Paint(painter,canvasColor);
		return;
	}

	painter.Clear(emColor::BLACK,canvasColor);
	canvasColor=emColor::BLACK;

	w=Mdl->GetWidth();
	h=Mdl->GetHeight();

	if (Mdl->IsBorderless()) {
		for (y=-1; y<=h; y++) {
			for (x=-1; x<=w; x++) {
				PaintPiece(
					painter,
					X0+x*DX,Y0+y*DY,DX,DY,
					Mdl->GetPiece(x,y),
					canvasColor
				);
			}
		}
		for (i=0; i<6; i++) {
			t=1.0-0.09*i;
			painter.PaintRectOutline(
				X0-DX+DX*t*0.5,
				Y0-DY+DY*t*0.5,
				DX*(w+2-t),
				DY*(h+2-t),
				DX*t,
				i==0 ? 0x00333399 : i==5 ? 0x000000FF :  0x00000044
			);
		}
	}
	else {
		for (y=0; y<h; y++) {
			for (x=0; x<w; x++) {
				PaintPiece(painter,X0+x*DX,Y0+y*DY,DX,DY,Mdl->GetPiece(x,y),canvasColor);
			}
		}
		t=0.2;
		painter.PaintRectOutline(
			X0-DX*t*0.5,
			Y0-DY*t*0.5,
			DX*(w+t),
			DY*(h+t),
			DX*t,
			0x666666FF,
			canvasColor
		);
	}

	if (Mdl->IsFinished()) {
		str=emString::Format(
			"Net complete!\n"
			"Penalty: %d",
			Mdl->GetPenaltyPoints()
		);
		t=EssenceW*0.003;
		painter.PaintTextBoxed(
			EssenceX+t,EssenceY+t,EssenceW,EssenceH,
			str,
			EssenceW/12,
			0x444400FF,0,EM_ALIGN_CENTER,EM_ALIGN_CENTER
		);
		painter.PaintTextBoxed(
			EssenceX,EssenceY,EssenceW,EssenceH,
			str,
			EssenceW/12,
			0xFFFF88FF,0,EM_ALIGN_CENTER,EM_ALIGN_CENTER
		);
	}
}


emPanel * emNetwalkPanel::CreateControlPanel(
	ParentArg parent, const emString & name
)
{
	if (HaveControlPanel) {
		return new emNetwalkControlPanel(parent,name,Mdl);
	}
	else {
		return emFilePanel::CreateControlPanel(parent,name);
	}
}


void emNetwalkPanel::PaintPiece(
	const emPainter & painter, double x, double y, double w, double h,
	int piece, emColor canvasColor
)
{
	emColor col;
	double cx,cy,x2,y2,t;

	t=w*0.01;
	x+=t*0.5;
	y+=t*0.5;
	w-=t;
	h-=t;
	painter.PaintRectOutline(x,y,w,h,t,0x666666FF,canvasColor);
	x+=t*0.5;
	y+=t*0.5;
	w-=t;
	h-=t;

	cx=x+w*0.5;
	cy=y+h*0.5;
	x2=x+w;
	y2=y+h;

	if (piece&emNetwalkModel::PF_MARKED) {
		col=0x777777FF;
		t=w*0.42;
		if (piece&emNetwalkModel::PF_EAST) {
			painter.PaintLine(
				cx,cy,x2,cy,t,
				emPainter::LC_ROUND,emPainter::LC_FLAT,col
			);
		}
		if (piece&emNetwalkModel::PF_SOUTH) {
			painter.PaintLine(
				cx,cy,cx,y2,t,
				emPainter::LC_ROUND,emPainter::LC_FLAT,col
			);
		}
		if (piece&emNetwalkModel::PF_WEST) {
			painter.PaintLine(
				cx,cy,x,cy,t,
				emPainter::LC_ROUND,emPainter::LC_FLAT,col
			);
		}
		if (piece&emNetwalkModel::PF_NORTH) {
			painter.PaintLine(
				cx,cy,cx,y,t,
				emPainter::LC_ROUND,emPainter::LC_FLAT,col
			);
		}
	}

	col=0xBBBBAAFF;
	t=w*0.2;
	if (piece&emNetwalkModel::PF_EAST) {
		painter.PaintLine(
			cx,cy,x2,cy,t,
			emPainter::LC_ROUND,emPainter::LC_FLAT,col
		);
	}
	if (piece&emNetwalkModel::PF_SOUTH) {
		painter.PaintLine(
			cx,cy,cx,y2,t,
			emPainter::LC_ROUND,emPainter::LC_FLAT,col
		);
	}
	if (piece&emNetwalkModel::PF_WEST) {
		painter.PaintLine(
			cx,cy,x,cy,t,
			emPainter::LC_ROUND,emPainter::LC_FLAT,col
		);
	}
	if (piece&emNetwalkModel::PF_NORTH) {
		painter.PaintLine(
			cx,cy,cx,y,t,
			emPainter::LC_ROUND,emPainter::LC_FLAT,col
		);
	}

	col=(piece&emNetwalkModel::PF_FILLED)?0x7788FFFF:0xAA9988FF;
	t=w*0.12;
	if (piece&emNetwalkModel::PF_EAST) {
		painter.PaintLine(
			cx,cy,x2,cy,t,
			emPainter::LC_ROUND,emPainter::LC_FLAT,col
		);
	}
	if (piece&emNetwalkModel::PF_SOUTH) {
		painter.PaintLine(
			cx,cy,cx,y2,t,
			emPainter::LC_ROUND,emPainter::LC_FLAT,col
		);
	}
	if (piece&emNetwalkModel::PF_WEST) {
		painter.PaintLine(
			cx,cy,x,cy,t,
			emPainter::LC_ROUND,emPainter::LC_FLAT,col
		);
	}
	if (piece&emNetwalkModel::PF_NORTH) {
		painter.PaintLine(
			cx,cy,cx,y,t,
			emPainter::LC_ROUND,emPainter::LC_FLAT,col
		);
	}

	if (piece&emNetwalkModel::PF_TARGET) {
		t=w*0.0008;
		painter.PaintRect(cx-180*t,cy+210*t,360*t, 90*t,0x000000FF);
		painter.PaintRect(cx-350*t,cy-320*t,700*t,550*t,0xFFFFEEFF);
		painter.PaintRect(cx-180*t,cy+235*t,360*t, 40*t,0xFFFFEEFF);
		painter.PaintRect(cx-235*t,cy+280*t,470*t, 40*t,0xFFFFEEFF);
		col=(piece&emNetwalkModel::PF_FILLED)?0x5088FFFF:0x443333FF;
		painter.PaintRect(cx-290*t,cy-260*t,580*t,400*t,col);
	}
	if (piece&emNetwalkModel::PF_SOURCE) {
		t=w*0.0008;
		painter.PaintRect(cx-230*t,cy+310*t,460*t, 45*t,0x000000FF);
		painter.PaintRect(cx-250*t,cy-370*t,500*t,700*t,0xFFFFEEFF);
		painter.PaintRect(cx-230*t,cy+335*t,460*t, 40*t,0xFFFFEEFF);
		painter.PaintRect(cx-210*t,cy-330*t, 80*t, 20*t,0x00AA00FF);
	}
}


void emNetwalkPanel::PrepareTransformation()
{
	int tw,th;
	double h;

	if (!IsViewed() || !IsVFSGood()) {
		EssenceX=0.0;
		EssenceY=0.0;
		EssenceW=1.0;
		EssenceH=GetHeight();
		X0=0.0;
		Y0=0.0;
		DX=1.0;
		DY=1.0;
	}
	else {
		h=GetHeight();
		tw=Mdl->GetWidth()+2;
		th=Mdl->GetHeight()+2;
		DX=1.0/tw;
		DY=h/th;
		if (DX>DY) DX=DY; else DY=DX;
		EssenceW=tw*DX;
		EssenceH=th*DY;
		EssenceX=(1.0-EssenceW)*0.5;
		EssenceY=(h-EssenceH)*0.5;
		X0=EssenceX+DX;
		Y0=EssenceY+DY;
	}
}
